@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002,2009 Red Hat, Inc.
+ * Copyright (C) 2002,2009,2010 Red Hat, Inc.
  *
  * This is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Library General Public License as published by
@@ -35,7 +35,7 @@ _vte_ring_validate (VteRing * ring)
 {
 	g_assert(ring != NULL);
 	_vte_debug_print(VTE_DEBUG_RING,
-			" Delta = %u, Length = %u, Max = %u, Writable = %u.\n",
+			" Delta = %lu, Length = %lu, Max = %lu, Writable = %lu.\n",
 			ring->start, ring->end - ring->start,
 			ring->max, ring->end - ring->writable);
 
@@ -67,7 +67,7 @@ _vte_ring_init (VteRing *ring, gulong max_rows)
 	ring->row_stream = _vte_file_stream_new ();
 
 	ring->last_attr.text_offset = 0;
-	ring->last_attr.attr.i = 0;
+	ring->last_attr.attr.i = basic_cell.i.attr;
 	ring->utf8_buffer = g_string_sized_new (128);
 
 	_vte_row_data_init (&ring->cached_row);
@@ -118,7 +118,6 @@ _vte_ring_freeze_row (VteRing *ring, gulong position, const VteRowData *row)
 	VteRowRecord record;
 	VteCell *cell;
 	GString *buffer = ring->utf8_buffer;
-	guint32 basic_attr = basic_cell.i.attr;
 	int i;
 
 	_vte_debug_print (VTE_DEBUG_RING, "Freezing row %lu.\n", position);
@@ -145,7 +144,6 @@ _vte_ring_freeze_row (VteRing *ring, gulong position, const VteRowData *row)
 		attr.s = cell->attr;
 		if (G_LIKELY (!attr.s.fragment)) {
 
-			attr.i ^= basic_attr;
 			if (ring->last_attr.attr.i != attr.i) {
 				ring->last_attr.text_offset = record.text_offset + buffer->len;
 				_vte_stream_append (ring->attr_stream, (const char *) &ring->last_attr, sizeof (ring->last_attr));
@@ -157,9 +155,7 @@ _vte_ring_freeze_row (VteRing *ring, gulong position, const VteRowData *row)
 
 			num_chars = _vte_unistr_strlen (cell->c);
 			if (num_chars > 1) {
-				attr.s = cell->attr;
 				attr.s.columns = 0;
-				attr.i ^= basic_attr;
 				ring->last_attr.text_offset = record.text_offset + buffer->len
 							    + g_unichar_to_utf8 (_vte_unistr_get_base (cell->c), NULL);
 				_vte_stream_append (ring->attr_stream, (const char *) &ring->last_attr, sizeof (ring->last_attr));
@@ -185,7 +181,6 @@ _vte_ring_thaw_row (VteRing *ring, gulong position, VteRowData *row, gboolean tr
 	VteCell cell;
 	const char *p, *q, *end;
 	GString *buffer = ring->utf8_buffer;
-	guint32 basic_attr = basic_cell.i.attr;
 
 	_vte_debug_print (VTE_DEBUG_RING, "Thawing row %lu.\n", position);
 
@@ -227,7 +222,6 @@ _vte_ring_thaw_row (VteRing *ring, gulong position, VteRowData *row, gboolean tr
 			attr = attr_change.attr;
 		}
 
-		attr.i ^= basic_attr;
 		cell.attr = attr.s;
 		cell.c = g_utf8_get_char (p);
 
@@ -260,7 +254,7 @@ _vte_ring_thaw_row (VteRing *ring, gulong position, VteRowData *row, gboolean tr
 		if (records[0].text_offset < ring->last_attr.text_offset)
 			if (!_vte_stream_read (ring->attr_stream, records[0].attr_offset, (char *) &ring->last_attr, sizeof (ring->last_attr))) {
 				ring->last_attr.text_offset = 0;
-				ring->last_attr.attr.i = 0;
+				ring->last_attr.attr.i = basic_cell.i.attr;
 			}
 		_vte_stream_truncate (ring->row_stream, position * sizeof (record));
 		_vte_stream_truncate (ring->attr_stream, records[0].attr_offset);
@@ -278,7 +272,7 @@ _vte_ring_reset_streams (VteRing *ring, gulong position)
 	_vte_stream_reset (ring->attr_stream, 0);
 
 	ring->last_attr.text_offset = 0;
-	ring->last_attr.attr.i = 0;
+	ring->last_attr.attr.i = basic_cell.i.attr;
 
 	ring->last_page = position;
 }
@@ -565,3 +559,73 @@ _vte_ring_append (VteRing * ring)
 	return _vte_ring_insert (ring, _vte_ring_next (ring));
 }
 
+
+static gboolean
+_vte_ring_write_row (VteRing *ring,
+		     GOutputStream *stream,
+		     VteRowData *row,
+		     VteTerminalWriteFlags flags,
+		     GCancellable *cancellable,
+		     GError **error)
+{
+	VteCell *cell;
+	GString *buffer = ring->utf8_buffer;
+	int i;
+	gsize bytes_written;
+
+	/* Simple version of the loop in _vte_ring_freeze_row().
+	 * TODO Should unify one day */
+	g_string_set_size (buffer, 0);
+	for (i = 0, cell = row->cells; i < row->len; i++, cell++) {
+		if (G_LIKELY (!cell->attr.fragment))
+			_vte_unistr_append_to_string (cell->c, buffer);
+	}
+	if (!row->attr.soft_wrapped)
+		g_string_append_c (buffer, '\n');
+
+	return g_output_stream_write_all (stream, buffer->str, buffer->len, &bytes_written, cancellable, error);
+}
+
+/**
+ * _vte_ring_write_contents:
+ * @ring: a #VteRing
+ * @stream: a #GOutputStream to write to
+ * @flags: a set of #VteTerminalWriteFlags
+ * @cancellable: optional #GCancellable object, %NULL to ignore
+ * @error: a #GError location to store the error occuring, or %NULL to ignore
+ *
+ * Write entire ring contents to @stream according to @flags.
+ *
+ * Return: %TRUE on success, %FALSE if there was an error
+ */
+gboolean
+_vte_ring_write_contents (VteRing *ring,
+			  GOutputStream *stream,
+			  VteTerminalWriteFlags flags,
+			  GCancellable *cancellable,
+			  GError **error)
+{
+	gulong i;
+
+	_vte_debug_print(VTE_DEBUG_RING, "Writing contents to GOutputStream.\n");
+
+	if (ring->start < ring->writable) {
+		VteRowRecord record;
+		/* XXX what to do in case of error? */
+		if (_vte_ring_read_row_record (ring, &record, ring->start)) {
+			if (!_vte_stream_write_contents (ring->text_stream, stream,
+							 record.text_offset,
+							 cancellable, error))
+				return FALSE;
+		}
+	}
+
+	for (i = ring->writable; i < ring->end; i++) {
+		if (!_vte_ring_write_row (ring, stream,
+					  _vte_ring_writable_index (ring, i),
+					  flags, cancellable, error))
+			return FALSE;
+	}
+
+	return TRUE;
+}
